@@ -1,6 +1,6 @@
 package Ubic::Cmd;
 BEGIN {
-  $Ubic::Cmd::VERSION = '1.21_01';
+  $Ubic::Cmd::VERSION = '1.32_01';
 }
 
 use strict;
@@ -214,9 +214,15 @@ sub traverse($$$) {
 sub print_status($$;$$) {
     my $self = _obj(shift);
     my $service = shift;
-    my $cached = shift;
+    my $force_cached = shift;
     my $results = shift || Ubic::Cmd::Results->new;
 
+    # TODO - use Credentials instead
+    my $user = getpwuid($>);
+    unless (defined $user) {
+        die "Can't detect user by uid $>";
+    }
+
     $self->traverse($service, sub {
         my $service = shift;
         my $name = $service->full_name;
@@ -229,8 +235,10 @@ sub print_status($$;$$) {
         }
 
         my $status;
-        if ($cached) {
+        my $cached;
+        if ($force_cached or ($> and $user ne Ubic->service($name)->user)) {
             $status = Ubic->cached_status($name);
+            $cached = 1;
         }
         else {
             $status = eval { Ubic->status($name) };
@@ -269,9 +277,6 @@ sub run {
     }
 
     my $command = $params->{command};
-    if ($command eq 'status' and $>) {
-        print "Not a root, printing cached statuses\n";
-    }
 
     my $results = Ubic::Cmd::Results->new;
     for my $name (@names) {
@@ -280,6 +285,27 @@ sub run {
     exit $results->exit_code;
 }
 
+sub _check_multiop {
+    my $self = _obj(shift);
+    my ($service, $command, $force) = validate_pos(@_, 1, 1, 1);
+
+    return unless $service->isa('Ubic::Multiservice');
+
+    my $screen_name = $service->name || 'root';
+    my $multiop = $service->multiop;
+    if ($multiop eq 'forbidden') {
+        die "$screen_name multiservice forbids $command\n";
+    }
+    elsif ($multiop eq 'protected') {
+        unless ($force) {
+            die "$screen_name is protected multiservice, specify --force if you know what you're doing\n";
+        }
+    }
+    elsif ($multiop ne 'allowed') {
+        die "$screen_name has strange multiop value '$multiop'\n";
+    }
+}
+
 # run and modify results object
 sub _run_impl {
     my $self = _obj(shift);
@@ -294,15 +320,12 @@ sub _run_impl {
     my $results = $params->{results};
 
     if ($command eq 'status' or $command eq 'cached-status') {
-        my $cached;
-        if ($command eq 'status' and $>) {
-            $cached = 1;
-        }
+        my $force_cached;
         if ($command eq 'cached-status') {
-            $cached = 1;
+            $force_cached = 1;
         }
         try {
-            $self->print_status($name, $cached, $results);
+            $self->print_status($name, $force_cached, $results);
         }
         catch {
             print STDERR $_;
@@ -321,25 +344,10 @@ sub _run_impl {
     # but we need to construct service to check it's custom commands
     my $service = $name ? Ubic->service($name) : Ubic->root_service;
 
-    if ($service->isa('Ubic::Multiservice')) {
-        my $screen_name = $name || 'root';
-        my $multiop = $service->multiop;
-        if ($multiop eq 'forbidden') {
-            die "$screen_name multiservice forbids $command";
-        }
-        elsif ($multiop eq 'protected') {
-            unless ($params->{force}) {
-                die "$screen_name is protected multiservice, specify --force if you know what you're doing";
-            }
-        }
-        elsif ($multiop ne 'allowed') {
-            die "$screen_name has strange multiop value '$multiop'";
-        }
-    }
-
     # yes, custom "start" command will override default "start" command, although it's not very useful :)
     # but we need this because of current "logrotate" hack
     if (grep { $_ eq $command } $service->custom_commands) {
+        $self->_check_multiop($service, $command, $params->{force});
         try {
             $self->do_custom_command($service, $command, $results);
         }
@@ -358,6 +366,7 @@ sub _run_impl {
         $self->usage($command);
     }
 
+    $self->_check_multiop($service, $command, $params->{force});
     try {
         $self->$method($service, $results);
     }
@@ -386,7 +395,7 @@ Ubic::Cmd - ubic methods with pretty printing.
 
 =head1 VERSION
 
-version 1.21_01
+version 1.32_01
 
 =head1 SYNOPSIS
 
@@ -447,11 +456,13 @@ WARNING: exits on invocation!
 
 Process each subservice of C<$name_or_service> with C<$callback>, printing correct indentations.
 
-=item B<< print_status($name, $cached_flag) >>
+=item B<< print_status($name, $force_cached) >>
 
-=item B<< print_status($service, $cached_flag) >>
+=item B<< print_status($name, $force_cached, $results) >>
 
-Print status of given service identified by name or by object. If C<$cached_flag> is true, prints status cached in watchdog file.
+Print status of given service identified by name or by object.
+
+If C<$force_cached> is true, prints status cached in watchdog file. Otherwise, check for real status if current user is B<root> or if current user is equal to service user.
 
 =item B<< run($params_hashref) >>
 
@@ -489,7 +500,7 @@ Vyacheslav Matyukhin <mmcleric@yandex-team.ru>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Yandex LLC.
+This software is copyright (c) 2011 by Yandex LLC.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
