@@ -1,6 +1,6 @@
 package Ubic::Daemon;
 BEGIN {
-  $Ubic::Daemon::VERSION = '1.21_01';
+  $Ubic::Daemon::VERSION = '1.32_01';
 }
 
 use strict;
@@ -11,11 +11,13 @@ use warnings;
 
 use IO::Handle;
 use POSIX qw(setsid);
+use Time::HiRes qw(sleep);
+use Params::Validate qw(:all);
+use Carp;
+
 use Ubic::Lockf;
 use Ubic::Daemon::Status;
-use Time::HiRes qw(sleep);
-
-use Carp;
+use Ubic::Daemon::PidState;
 
 use parent qw(Exporter);
 our @EXPORT_OK = qw(start_daemon stop_daemon check_daemon);
@@ -23,120 +25,21 @@ our %EXPORT_TAGS = (
     all => \@EXPORT_OK,
 );
 
-use Params::Validate qw(:all);
+our $OS;
+sub import {
+    my %module = (
+        linux   => 'Linux',
+    );
 
-# get pid's guid
-# returns undef if pid not found, throws exception on other errors
-sub _pid2guid($) {
-    my $pid = shift;
-    unless (-d "/proc/$pid") {
-        return; # process not found
-    }
-    my $opened = open(my $fh, '<', "/proc/$pid/stat");
-    unless ($opened) {
-        # open failed
-        my $error = $!;
-        unless (-d "/proc/$pid") {
-            return; # process exited right now
-        }
-        die "Open /proc/$pid/stat failed: $!";
-    }
-    my $line = <$fh>;
-    my @fields = split /\s+/, $line;
-    my $guid = $fields[21];
-    return $guid;
-}
+    # UBIC_DAEMON_OS support is here only for tests
+    my $module = $ENV{UBIC_DAEMON_OS} || $module{$^O} || 'POSIX';
 
-# this method should be called only when pidfile is locked (check before removing?)
-sub _remove_pidfile($) {
-    my ($file) = validate_pos(@_, { type => SCALAR });
-    if (-d $file) {
-        if (-e "$file/pid") {
-            unlink "$file/pid" or die "Can't remove $file/pid: $!";
-        }
-    }
-    else {
-        unlink $file or die "Can't remove $file: $!";
-    }
-}
-
-sub _lock_pidfile($;$) {
-    my ($pidfile, $timeout) = validate_pos(@_, { type => SCALAR }, 0);
-    $timeout ||= 0;
-    if (-d $pidfile) {
-        # new-style pidfile
-        return lockf("$pidfile/lock", { timeout => $timeout });
-    }
-    else {
-        return lockf($pidfile, { blocking => 0 });
-    }
-}
-
-# write guardian pid and guid to pidfile
-sub _write_pidfile($$) {
-    my $file = shift;
-    unless (-d $file) {
-        die "piddir $file not exists";
-    }
-    my $params = validate(@_, {
-        pid => 1,
-        guid => 1,
-    });
-    my ($pid, $guid) = @$params{qw/ pid guid /};
-    my $self_pid = $$;
-    open my $fh, '>', "$file/pid.new" or die "Can't write '$file/pid.new': $!";
-    print {$fh} "pid $self_pid\n";
-    print {$fh} "guid $guid\n";
-    print {$fh} "daemon $pid\n";
-    $fh->flush;
-    close $fh or die "Can't close '$file/pid.new': $!";
-    rename "$file/pid.new" => "$file/pid" or die "Can't commit pidfile $file: $!";
-}
-
-# read daemon info from pidfile
-# returns undef if pidfile not found
-# throws exceptions when content is invalid
-sub _read_pidfile($) {
-    my ($file) = validate_pos(@_, { type => SCALAR });
-    my $content;
-    my $parse_content = sub {
-        if ($content =~ /\A pid \s+ (\d+) \n guid \s+ (\d+) (?: \n daemon \s+ (\d+) )? \Z/x) {
-            # new format
-            return { pid => $1, guid => $2, daemon => $3, format => 'new' };
-        }
-        else {
-            die "invalid pidfile content in pidfile $file";
-        }
-    };
-    if (-d $file) {
-        # pidfile as dir
-        my $open_success = open my $fh, '<', "$file/pid";
-        unless ($open_success) {
-            if ($!{ENOENT}) {
-                return; # pidfile not found, daemon is not running
-            }
-            else {
-                die "Failed to open '$file/pid': $!";
-            }
-        }
-        $content = join '', <$fh>;
-        return $parse_content->();
-    }
-    else {
-        # deprecated - single pidfile without piddir
-        if (-f $file and not -s $file) {
-            return; # empty pidfile - old way to stop services
-        }
-        open my $fh, '<', $file or die "Failed to open $file: $!";
-        $content = join '', <$fh>;
-        if ($content =~ /\A (\d+) \Z/x) {
-            # old format
-            return { pid => $1, format => 'old' };
-        }
-        else {
-            return $parse_content->();
-        }
+    require "Ubic/Daemon/OS/$module.pm";
+    $OS = eval "Ubic::Daemon::OS::$module->new";
+    unless ($OS) {
+        die "failed to initialize OS-specific module $module: $@";
     }
+    __PACKAGE__->export_to_level(1, @_);
 }
 
 sub _log {
@@ -152,10 +55,11 @@ sub stop_daemon($;@) {
     });
     my $timeout = $options->{timeout} if defined $options->{timeout};
 
-    if (not -d $pidfile and not -s $pidfile) {
-        return 'not running';
-    }
-    my $piddata = _read_pidfile($pidfile);
+    # TODO - move this check into Ubic::Daemon::PidState
+    my $pid_state = Ubic::Daemon::PidState->new($pidfile);
+    return 'not running' if $pid_state->is_empty;
+
+    my $piddata = $pid_state->read;
     unless ($piddata) {
         return 'not running';
     }
@@ -218,13 +122,9 @@ sub start_daemon($) {
     if (check_daemon($pidfile)) {
         croak "Daemon with pidfile $pidfile already running, can't start";
     }
-    if (-e $pidfile and not -d $pidfile) {
-        print "converting $pidfile to dir\n";
-        unlink $pidfile or die "Can't unlink $pidfile: $!";
-    }
-    unless (-d $pidfile) {
-        mkdir $pidfile or die "Can't create $pidfile: $!";
-    }
+
+    my $pid_state = Ubic::Daemon::PidState->new($pidfile);
+    $pid_state->init;
 
     my $stdin = '/dev/null';
 
@@ -258,13 +158,11 @@ sub start_daemon($) {
             }
 
             # Close all inherited filehandles except $write_pipe (it will be closed explicitly).
-            # Do not close fh if uses 'function' option instead of 'bin' ('function' should be deprecated).
+            # Do not close fh if uses 'function' option instead of 'bin'
+            # ('function' support should be removed altogether because of this, actually; it's evil).
             if ($bin) {
-                my @fd_nums = map { s!^.*/!!; $_ } glob("/proc/$$/fd/*");
                 my $write_pipe_fd_num = fileno($write_pipe);
-                foreach (@fd_nums) {
-                    POSIX::close($_) if ($_ != $write_pipe_fd_num);
-                }
+                $OS->close_all_fh($write_pipe_fd_num); # except pipe
             }
 
             open STDOUT, ">>", $stdout or die "Can't write to '$stdout': $!";
@@ -285,9 +183,9 @@ sub start_daemon($) {
             # There should be no races when Ubic::Daemon is used in context of
             # ubic service, because services has additional lock, but
             # Ubic::Daemon can be useful without services as well.
-            $lock = _lock_pidfile($pidfile, 5) or die "Can't lock $pidfile";
+            $lock = $pid_state->lock(5) or die "Can't lock $pid_state";
 
-            _remove_pidfile($pidfile);
+            $pid_state->remove;
             _log($ubic_fh, "got lock");
 
             if (defined $user) {
@@ -302,8 +200,8 @@ sub start_daemon($) {
             if ($child = fork) {
                 # guardian
 
-                my $child_guid = _pid2guid($child);
-                _write_pidfile($pidfile, { pid => $child, guid => $child_guid });
+                my $child_guid = $OS->pid2guid($child);
+                $pid_state->write({ pid => $child, guid => $child_guid });
 
                 _log($ubic_fh, "guardian pid: $$");
                 _log($ubic_fh, "daemon pid: $child");
@@ -315,7 +213,7 @@ sub start_daemon($) {
                     _log($ubic_fh, "sending SIGKILL to $child");
                     kill -9 => $child;
                     _log($ubic_fh, "child probably killed by SIGKILL");
-                    _remove_pidfile($pidfile);
+                    $pid_state->remove();
                     $instant_exit->(0);
                 };
 
@@ -344,7 +242,7 @@ sub start_daemon($) {
                         if ($sigterm_sent && $signal == &POSIX::SIGTERM) {
                             # it's ok, we probably sent this signal ourselves
                             _log($ubic_fh, "daemon exited by sigterm");
-                            _remove_pidfile($pidfile);
+                            $pid_state->remove;
                             $instant_exit->(0);
                         }
                         $msg = "Daemon $child failed with signal $signal";
@@ -353,11 +251,11 @@ sub start_daemon($) {
                         $msg = "Daemon failed: $?";
                     }
                     _log($ubic_fh, $msg);
-                    _remove_pidfile($pidfile);
+                    $pid_state->remove;
                     $instant_exit->(1);
                 }
                 _log($ubic_fh, "daemon exited");
-                _remove_pidfile($pidfile);
+                $pid_state->remove;
                 $instant_exit->(0);
             }
             else {
@@ -409,15 +307,12 @@ sub start_daemon($) {
 
 sub check_daemon {
     my ($pidfile) = @_;
-    if (not -d $pidfile and not -s $pidfile) {
-        return undef; # empty, old-style pidfile
-    }
-    if (-d $pidfile and not -e "$pidfile/pid") {
-        return undef;
-    }
 
-    my $lock = _lock_pidfile($pidfile);
-    my $piddata = _read_pidfile($pidfile);
+    my $pid_state = Ubic::Daemon::PidState->new($pidfile);
+    return undef if $pid_state->is_empty;
+
+    my $lock = $pid_state->lock;
+    my $piddata = $pid_state->read;
     unless ($lock) {
         # locked => daemon is alive
         return Ubic::Daemon::Status->new({ pid => $piddata->{daemon} });
@@ -436,34 +331,31 @@ sub check_daemon {
         use Data::Dumper;
         die "pidfile $pidfile exists, but daemon pid is not saved in it, so existing unguarded daemon can't be killed (piddata: ".Dumper($piddata).")";
     }
-    unless (-d "/proc/$piddata->{daemon}") {
-        _remove_pidfile($pidfile);
+    unless ($OS->pid_exists($piddata->{daemon})) {
+        $pid_state->remove;
         print "pidfile $pidfile removed - daemon with cached pid $piddata->{daemon} not found\n";
         return undef;
     }
-    open my $daemon_cmd_fh, '<', "/proc/$piddata->{daemon}/cmdline" or die "Can't open daemon's cmdline: $!";
-    my $daemon_cmd = <$daemon_cmd_fh>;
-    $daemon_cmd =~ s/\x{00}$//;
-    $daemon_cmd =~ s/\x{00}/ /g;
-    close $daemon_cmd_fh;
 
-    my @procdir_stat = stat("/proc/$piddata->{daemon}");
-    my $guid = _pid2guid($piddata->{daemon});
+    # TODO - wrap in eval and return undef if pid2cmd fails?
+    my $daemon_cmd = $OS->pid2cmd($piddata->{daemon});
+
+    my $guid = $OS->pid2guid($piddata->{daemon});
     unless ($guid) {
         print "daemon '$daemon_cmd' from $pidfile just disappeared\n";
         return undef;
     }
     if ($guid eq $piddata->{guid}) {
-        warn "killing unguarded daemon '$daemon_cmd' with pid $piddata->{daemon} from $pidfile\n";
+        print "killing unguarded daemon '$daemon_cmd' with pid $piddata->{daemon} from $pidfile\n";
         kill -9 => $piddata->{daemon};
-        _remove_pidfile($pidfile);
+        $pid_state->remove;
         print "pidfile $pidfile removed\n";
         return undef;
     }
     print "daemon pid $piddata->{daemon} cached in pidfile $pidfile, ubic-guardian not found\n";
     print "current process '$daemon_cmd' with that pid looks too fresh and will not be killed\n";
     print "pidfile $pidfile removed\n";
-    _remove_pidfile($pidfile);
+    $pid_state->remove;
     return undef;
 }
 
@@ -479,7 +371,7 @@ Ubic::Daemon - toolkit for creating daemonized process
 
 =head1 VERSION
 
-version 1.21_01
+version 1.32_01
 
 =head1 SYNOPSIS
 
@@ -613,7 +505,7 @@ Vyacheslav Matyukhin <mmcleric@yandex-team.ru>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Yandex LLC.
+This software is copyright (c) 2011 by Yandex LLC.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
