@@ -1,6 +1,6 @@
 package Ubic::Multiservice::Dir;
 BEGIN {
-  $Ubic::Multiservice::Dir::VERSION = '1.21_01';
+  $Ubic::Multiservice::Dir::VERSION = '1.32_01';
 }
 
 use strict;
@@ -20,13 +20,6 @@ sub new($$) {
     return bless { service_dir => $dir } => $class;
 }
 
-sub set_service_dir {
-    my $self = shift;
-    my $dir = shift;
-    $self->{service_dir} = $dir; # TODO - check that dir exists?
-}
-
-
 sub has_simple_service($$) {
     my $self = shift;
     my ($name) = validate_pos(@_, {type => SCALAR, regex => qr/^[\w.-]+$/});
@@ -101,7 +94,7 @@ Ubic::Multiservice::Dir - multiservice which uses directory with configs to inst
 
 =head1 VERSION
 
-version 1.21_01
+version 1.32_01
 
 =head1 METHODS
 
@@ -111,12 +104,6 @@ version 1.21_01
 
 Constructor.
 
-=item B<< set_service_dir($dir) >>
-
-Set service dir in runtime.
-
-This class can be mixin to another service, in this case constructor will never be invoked.
-
 =back
 
 =head1 SEE ALSO
@@ -131,7 +118,7 @@ Vyacheslav Matyukhin <mmcleric@yandex-team.ru>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Yandex LLC.
+This software is copyright (c) 2011 by Yandex LLC.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
