@@ -1,6 +1,6 @@
 package Ubic::Persistent;
 BEGIN {
-  $Ubic::Persistent::VERSION = '1.21_01';
+  $Ubic::Persistent::VERSION = '1.32_01';
 }
 
 use strict;
@@ -11,6 +11,7 @@ use warnings;
 
 use JSON;
 use Ubic::Lockf;
+use Ubic::AtomicFile;
 
 {
     # JSON.pm v2 incompatibility with v1 is really, really annoying.
@@ -64,11 +65,8 @@ sub new {
 sub commit {
     my $self = shift;
     my $fname = $meta->{$self}{fname};
-    open my $tmp_fh, '>', "$fname.new" or die "Can't write '$fname.new': $!";
 
-    print {$tmp_fh} objToJson({ %$self });
-    close $tmp_fh or die "Can't write to '$fname.new': $!";
-    rename "$fname.new" => $fname or die "Can't rename '$fname.new' to '$fname': $!";
+    Ubic::AtomicFile::store(objToJson({ %$self }) => $fname);
 }
 
 sub DESTROY {
@@ -89,7 +87,7 @@ Ubic::Persistent - simple hash-to-file persistence object
 
 =head1 VERSION
 
-version 1.21_01
+version 1.32_01
 
 =head1 SYNOPSIS
 
@@ -126,7 +124,7 @@ Vyacheslav Matyukhin <mmcleric@yandex-team.ru>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Yandex LLC.
+This software is copyright (c) 2011 by Yandex LLC.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
