@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 package Ubic::Ping;
 BEGIN {
-  $Ubic::Ping::VERSION = '1.21_01';
+  $Ubic::Ping::VERSION = '1.32_01';
 }
 
 use strict;
@@ -29,7 +29,7 @@ sub _print_status($;$) {
         print "disabled\n";
         return;
     }
-    my $status = Ubic->cached_status($name); # should read status from static file on disk
+    my $status = Ubic->cached_status($name)->status; # should read status from static file on disk
     if ($status eq 'running') {
         print "HTTP/1.0 200 OK\r\n\r\n";
         print "ok\n";
@@ -95,7 +95,7 @@ sub handle_request {
         }
         else {
             print "HTTP/1.0 404 Not found\r\n\r\n";
-            print "Expected /status/name/NAME or /status/port/PORT query\n";
+            print "Expected /status/service/NAME or /status/port/PORT query\n";
             return;
         }
     }
@@ -116,7 +116,7 @@ Ubic::Ping - http server which returns service status by it's name or port
 
 =head1 VERSION
 
-version 1.21_01
+version 1.32_01
 
 =head1 AUTHOR
 
@@ -124,7 +124,7 @@ Vyacheslav Matyukhin <mmcleric@yandex-team.ru>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Yandex LLC.
+This software is copyright (c) 2011 by Yandex LLC.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
