@@ -1,6 +1,6 @@
 package Ubic::PortMap;
 BEGIN {
-  $Ubic::PortMap::VERSION = '1.21_01';
+  $Ubic::PortMap::VERSION = '1.32_01';
 }
 
 use strict;
@@ -9,15 +9,15 @@ use warnings;
 # ABSTRACT: update and read mapping of ports to service names.
 
 
+use Try::Tiny;
+use Params::Validate qw(:all);
+
 use Ubic::Logger;
 use Ubic::Persistent;
 use Ubic;
-use Try::Tiny;
-
-use Params::Validate qw(:all);
 
 sub _portmap_file {
-    my $ubic_dir = $ENV{UBIC_DIR} || '/var/lib/ubic';
+    my $ubic_dir = Ubic->get_data_dir;
     my $PORTMAP_FILE = $ubic_dir.'/portmap';
     return $PORTMAP_FILE;
 }
@@ -86,14 +86,14 @@ Ubic::PortMap - update and read mapping of ports to service names.
 
 =head1 VERSION
 
-version 1.21_01
+version 1.32_01
 
 =head1 SYNOPSIS
 
     use Ubic::PortMap;
 
     Ubic::PortMap::update();
-    print Ubic::PortMap::port2name(12345); # ubic-ping
+    print Ubic::PortMap::port2name(12345); # ubic.ping
 
 =head1 METHODS
 
@@ -117,7 +117,7 @@ Vyacheslav Matyukhin <mmcleric@yandex-team.ru>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Yandex LLC.
+This software is copyright (c) 2011 by Yandex LLC.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
