@@ -1,6 +1,6 @@
 package Ubic::Result::Class;
 BEGIN {
-  $Ubic::Result::Class::VERSION = '1.21_01';
+  $Ubic::Result::Class::VERSION = '1.32_01';
 }
 
 use strict;
@@ -29,6 +29,7 @@ sub new {
     my $self = validate(@_, {
         type => { type => SCALAR, optional => 1 },
         msg => { optional => 1 },
+        cached => { optional => 1 },
     });
     $self->{type} ||= 'unknown';
     return bless $self => $class;
@@ -62,16 +63,17 @@ sub action {
 
 sub as_string {
     my $self = shift;
+    my $cached_str = ($self->{cached} ? ' [cached]' : '');
     if (defined $self->{msg}) {
         if ($self->{type} eq 'unknown') {
             return "$self->{msg}\n";
         }
         else {
-            return "$self->{type} ($self->{msg})";
+            return "$self->{type} ($self->{msg})".$cached_str;
         }
     }
     else {
-        return $self->type;
+        return $self->type.$cached_str;
     }
 }
 
@@ -87,7 +89,7 @@ Ubic::Result::Class - ubic result object
 
 =head1 VERSION
 
-version 1.21_01
+version 1.32_01
 
 =head1 SYNOPSIS
 
@@ -175,7 +177,7 @@ Vyacheslav Matyukhin <mmcleric@yandex-team.ru>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Yandex LLC.
+This software is copyright (c) 2011 by Yandex LLC.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
