@@ -1,11 +1,13 @@
 package Ubic::Service;
 BEGIN {
-  $Ubic::Service::VERSION = '1.21_01';
+  $Ubic::Service::VERSION = '1.32_01';
 }
 
 use strict;
 use warnings;
 
+use Ubic::Settings;
+
 # ABSTRACT: interface and base class for any ubic service
 
 
@@ -36,21 +38,11 @@ sub port {
 
 sub user {
     my $self = shift;
-    return 'root';
+    return Ubic::Settings->default_user;
 }
 
 sub group {
-    my $self = shift;
-    my $user = $self->user;
-    my $main_group = getgrgid((getpwnam $user)[3]);
-    setgrent();
-    my @groups;
-    while (my @grent = getgrent()) {
-        my @users = split / /, $grent[3];
-        push @groups, $grent[0] if grep { $_ eq $user } @users;
-    }
-    endgrent();
-    return ($main_group, @groups);
+    return ();
 }
 
 sub check_period {
@@ -59,7 +51,7 @@ sub check_period {
 }
 
 sub check_timeout {
-    return 600;
+    return 60;
 }
 
 sub custom_commands {
@@ -104,7 +96,6 @@ sub parent_name($;$) {
 
 1;
 
-
 __END__
 =pod
 
@@ -114,7 +105,7 @@ Ubic::Service - interface and base class for any ubic service
 
 =head1 VERSION
 
-version 1.21_01
+version 1.32_01
 
 =head1 SYNOPSIS
 
@@ -180,7 +171,7 @@ Default is C<undef>.
 
 Should return user from which the service can be controlled and will be running.
 
-Default is C<root>.
+Defaults to C<default_user> from L<Ubic::Settings> (i.e., C<root> for system-wide installations, or to the installation owner for local installations).
 
 =item B<group()>
 
@@ -188,7 +179,7 @@ Get list of groups from which the service can be controlled and will be running.
 
 First group from list will be used as real and effective group id, and other groups will be set as supplementary groups.
 
-Default is list of all groups of user as returned by C<user()> method.
+Default is an empty list, which will later be interpreted as default group of the user returned by C<user()> method.
 
 =item B<check_period()>
 
@@ -204,7 +195,7 @@ This parameter exists as a precaution against incorrectly implemented C<status()
 
 This parameter is *not* a timeout for querying your service by HTTP or whatever your status check is. Service-specific timeouts should be configured by other means.
 
-Default value is 600 seconds. It should not be changed unless you have a very good reason to do so (i.e., your service is so horribly slow that it can't start in 10 minutes).
+Default value is 60 seconds. It should not be changed unless you have a very good reason to do so (i.e., your service is so horribly slow that it can't start in 1 minute).
 
 =back
 
@@ -281,7 +272,7 @@ Vyacheslav Matyukhin <mmcleric@yandex-team.ru>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Yandex LLC.
+This software is copyright (c) 2011 by Yandex LLC.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
