@@ -1,22 +1,38 @@
 package Ubic::Service::SimpleDaemon;
 BEGIN {
-  $Ubic::Service::SimpleDaemon::VERSION = '1.21_01';
+  $Ubic::Service::SimpleDaemon::VERSION = '1.32_01';
 }
 
 use strict;
 use warnings;
 
-# ABSTRACT: variant of service when your service is simple daemonized binary
+# ABSTRACT: declarative service for daemonizing any binary
 
 
 use parent qw(Ubic::Service::Skeleton);
 
+use Cwd;
 use Ubic::Daemon qw(start_daemon stop_daemon check_daemon);
 use Ubic::Result qw(result);
+use Ubic::Settings;
+use File::Spec;
 
 use Params::Validate qw(:all);
 
-our $PID_DIR = $ENV{UBIC_DAEMON_PID_DIR} || "/var/lib/ubic/simple-daemon/pid";
+# Beware - this code will ignore any overrides if you're using custom Ubic->new(...) objects
+our $PID_DIR;
+
+sub _pid_dir {
+    return $PID_DIR if defined $PID_DIR;
+    if ($ENV{UBIC_DAEMON_PID_DIR}) {
+        warn "UBIC_DAEMON_PID_DIR env variable is deprecated, use Ubic->set_data_dir or configs instead (see Ubic::Settings for details)";
+        $PID_DIR = $ENV{UBIC_DAEMON_PID_DIR};
+    }
+    else {
+        $PID_DIR = Ubic::Settings->data_dir."/simple-daemon/pid";
+    }
+    return $PID_DIR;
+}
 
 sub new {
     my $class = shift;
@@ -28,6 +44,8 @@ sub new {
         stdout => { type => SCALAR, optional => 1 },
         stderr => { type => SCALAR, optional => 1 },
         ubic_log => { type => SCALAR, optional => 1 },
+        cwd => { type => SCALAR, optional => 1 },
+        env => { type => HASHREF, optional => 1 },
     });
 
     return bless {%$params} => $class;
@@ -36,11 +54,25 @@ sub new {
 sub pidfile {
     my ($self) = @_;
     my $name = $self->full_name or die "Can't start nameless SimpleDaemon";
-    return "$PID_DIR/$name";
+    return _pid_dir."/$name";
 }
 
 sub start_impl {
     my ($self) = @_;
+
+    my $old_cwd;
+    if (defined $self->{cwd}) {
+        $old_cwd = getcwd;
+        chdir $self->{cwd} or die "chdir to '$self->{cwd}' failed: $!";
+    }
+
+    local %ENV = %ENV;
+    if (defined $self->{env}) {
+        for my $key (keys %{ $self->{env} }) {
+            $ENV{$key} = $self->{env}{$key};
+        }
+    }
+
     my $start_params = {
         pidfile => $self->pidfile,
         bin => $self->{bin},
@@ -48,10 +80,17 @@ sub start_impl {
         stderr => $self->{stderr} || "/dev/null",
         ubic_log => $self->{ubic_log} || "/dev/null",
     };
-    if ($self->{user}) {
-        $start_params->{user} = $self->{user}; # do we actually need this? Ubic.pm should call setuid for us...
+    if ($old_cwd) {
+        for my $key (qw/ pidfile stdout stderr ubic_log /) {
+            next unless defined $start_params->{$key};
+            $start_params->{$key} = File::Spec->rel2abs($start_params->{$key}, $old_cwd);
+        }
     }
     start_daemon($start_params);
+
+    if (defined $old_cwd) {
+        chdir $old_cwd or die "chdir to '$old_cwd' failed: $!";
+    }
 }
 
 sub user {
@@ -86,29 +125,33 @@ sub status_impl {
 
 1;
 
-
 __END__
 =pod
 
 =head1 NAME
 
-Ubic::Service::SimpleDaemon - variant of service when your service is simple daemonized binary
+Ubic::Service::SimpleDaemon - declarative service for daemonizing any binary
 
 =head1 VERSION
 
-version 1.21_01
+version 1.32_01
 
 =head1 SYNOPSIS
 
     use Ubic::Service::SimpleDaemon;
-    my $service = Ubic::Service::SimpleDaemon->new({
-        name => "sleep",
+    my $service = Ubic::Service::SimpleDaemon->new(
         bin => "sleep 1000",
-    });
+        stdout => "/var/log/sleep.log",
+        stderr => "/var/log/sleep.err.log",
+        ubic_log => "/var/log/sleep.ubic.log",
+        user => "nobody",
+    );
 
 =head1 DESCRIPTION
 
-Unlike L<Ubic::Service::Common>, this class allows you to specify only name and binary of your service.
+Use this class to daemonize any binary.
+
+This module uses L<Ubic::Daemon> module for process daemonization. All pidfiles are stored in ubic data dir, with their names based on service names.
 
 =head1 METHODS
 
@@ -126,12 +169,6 @@ Parameters:
 
 Daemon binary.
 
-=item I<name>
-
-Service's name.
-
-Optional, will usually be set by upper-level multiservice. Don't set it unless you know what you're doing.
-
 =item I<user>
 
 User under which daemon will be started. Optional, default is C<root>.
@@ -150,6 +187,12 @@ File into which daemon's stdout will be redirected. Default is C</dev/null>.
 
 File into which daemon's stderr will be redirected. Default is C</dev/null>.
 
+=item I<name>
+
+Service's name.
+
+Optional, will usually be set by upper-level multiservice. Don't set it unless you know what you're doing.
+
 =back
 
 =item B<< pidfile() >>
@@ -168,7 +211,7 @@ Vyacheslav Matyukhin <mmcleric@yandex-team.ru>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Yandex LLC.
+This software is copyright (c) 2011 by Yandex LLC.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
